// flow-typed signature: b71fdca1c745c065f87e7e0172c64d49
// flow-typed version: <<STUB>>/@babel/preset-env_v^7.0.0-beta.34/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   '@babel/preset-env'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "@babel/preset-env" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "@babel/preset-env/data/built-in-features" {
  declare module.exports: any;
}

declare module "@babel/preset-env/data/plugin-features" {
  declare module.exports: any;
}

declare module "@babel/preset-env/data/shipped-proposals" {
  declare module.exports: any;
}

declare module "@babel/preset-env/data/unreleased-labels" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/available-plugins" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/built-in-definitions" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/debug" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/default-includes" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/index" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/module-transformations" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/normalize-options" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/targets-parser" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/use-built-ins-entry-plugin" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/use-built-ins-plugin" {
  declare module.exports: any;
}

declare module "@babel/preset-env/lib/utils" {
  declare module.exports: any;
}

// Filename aliases
declare module "@babel/preset-env/data/built-in-features.js" {
  declare module.exports: $Exports<"@babel/preset-env/data/built-in-features">;
}
declare module "@babel/preset-env/data/plugin-features.js" {
  declare module.exports: $Exports<"@babel/preset-env/data/plugin-features">;
}
declare module "@babel/preset-env/data/shipped-proposals.js" {
  declare module.exports: $Exports<"@babel/preset-env/data/shipped-proposals">;
}
declare module "@babel/preset-env/data/unreleased-labels.js" {
  declare module.exports: $Exports<"@babel/preset-env/data/unreleased-labels">;
}
declare module "@babel/preset-env/lib/available-plugins.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/available-plugins">;
}
declare module "@babel/preset-env/lib/built-in-definitions.js" {
  declare module.exports: $Exports<
    "@babel/preset-env/lib/built-in-definitions"
  >;
}
declare module "@babel/preset-env/lib/debug.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/debug">;
}
declare module "@babel/preset-env/lib/default-includes.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/default-includes">;
}
declare module "@babel/preset-env/lib/index.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/index">;
}
declare module "@babel/preset-env/lib/module-transformations.js" {
  declare module.exports: $Exports<
    "@babel/preset-env/lib/module-transformations"
  >;
}
declare module "@babel/preset-env/lib/normalize-options.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/normalize-options">;
}
declare module "@babel/preset-env/lib/targets-parser.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/targets-parser">;
}
declare module "@babel/preset-env/lib/use-built-ins-entry-plugin.js" {
  declare module.exports: $Exports<
    "@babel/preset-env/lib/use-built-ins-entry-plugin"
  >;
}
declare module "@babel/preset-env/lib/use-built-ins-plugin.js" {
  declare module.exports: $Exports<
    "@babel/preset-env/lib/use-built-ins-plugin"
  >;
}
declare module "@babel/preset-env/lib/utils.js" {
  declare module.exports: $Exports<"@babel/preset-env/lib/utils">;
}
