// flow-typed signature: e755d3e0a9b419028ff5dd7a95ff63cb
// flow-typed version: <<STUB>>/eslint-plugin-mocha_v^4.11.0/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'eslint-plugin-mocha'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "eslint-plugin-mocha" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "eslint-plugin-mocha/lib/rules/handle-done-callback" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/max-top-level-suites" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-exclusive-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-global-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-hooks-for-single-case" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-hooks" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-identical-title" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-mocha-arrows" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-nested-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-pending-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-return-and-callback" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-sibling-hooks" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-skipped-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-synchronous-tests" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/no-top-level-hooks" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/valid-suite-description" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/rules/valid-test-description" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/util/ast" {
  declare module.exports: any;
}

declare module "eslint-plugin-mocha/lib/util/settings" {
  declare module.exports: any;
}

// Filename aliases
declare module "eslint-plugin-mocha/index" {
  declare module.exports: $Exports<"eslint-plugin-mocha">;
}
declare module "eslint-plugin-mocha/index.js" {
  declare module.exports: $Exports<"eslint-plugin-mocha">;
}
declare module "eslint-plugin-mocha/lib/rules/handle-done-callback.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/handle-done-callback"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/max-top-level-suites.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/max-top-level-suites"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-exclusive-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-exclusive-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-global-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-global-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-hooks-for-single-case.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-hooks-for-single-case"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-hooks.js" {
  declare module.exports: $Exports<"eslint-plugin-mocha/lib/rules/no-hooks">;
}
declare module "eslint-plugin-mocha/lib/rules/no-identical-title.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-identical-title"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-mocha-arrows.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-mocha-arrows"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-nested-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-nested-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-pending-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-pending-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-return-and-callback.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-return-and-callback"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-sibling-hooks.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-sibling-hooks"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-skipped-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-skipped-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-synchronous-tests.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-synchronous-tests"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/no-top-level-hooks.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/no-top-level-hooks"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/valid-suite-description.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/valid-suite-description"
  >;
}
declare module "eslint-plugin-mocha/lib/rules/valid-test-description.js" {
  declare module.exports: $Exports<
    "eslint-plugin-mocha/lib/rules/valid-test-description"
  >;
}
declare module "eslint-plugin-mocha/lib/util/ast.js" {
  declare module.exports: $Exports<"eslint-plugin-mocha/lib/util/ast">;
}
declare module "eslint-plugin-mocha/lib/util/settings.js" {
  declare module.exports: $Exports<"eslint-plugin-mocha/lib/util/settings">;
}
