// flow-typed signature: b6e0afc54f725388765e4f0822993a69
// flow-typed version: <<STUB>>/@babel/preset-flow_v^7.0.0-beta.34/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   '@babel/preset-flow'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "@babel/preset-flow" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "@babel/preset-flow/lib/index" {
  declare module.exports: any;
}

// Filename aliases
declare module "@babel/preset-flow/lib/index.js" {
  declare module.exports: $Exports<"@babel/preset-flow/lib/index">;
}
