// flow-typed signature: a094bc926ba20898ac8fc73f62893cf8
// flow-typed version: <<STUB>>/jest-diff_v^21.2.1/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'jest-diff'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "jest-diff" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "jest-diff/build/constants" {
  declare module.exports: any;
}

declare module "jest-diff/build/diff_strings" {
  declare module.exports: any;
}

declare module "jest-diff/build/index" {
  declare module.exports: any;
}

// Filename aliases
declare module "jest-diff/build/constants.js" {
  declare module.exports: $Exports<"jest-diff/build/constants">;
}
declare module "jest-diff/build/diff_strings.js" {
  declare module.exports: $Exports<"jest-diff/build/diff_strings">;
}
declare module "jest-diff/build/index.js" {
  declare module.exports: $Exports<"jest-diff/build/index">;
}
