// flow-typed signature: d6c8a9ce587a3d319819825267e4b219
// flow-typed version: <<STUB>>/eslint-plugin-flowtype-errors_v^3.3.6/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'eslint-plugin-flowtype-errors'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "eslint-plugin-flowtype-errors" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "eslint-plugin-flowtype-errors/dist/collect" {
  declare module.exports: any;
}

declare module "eslint-plugin-flowtype-errors/dist/filter" {
  declare module.exports: any;
}

declare module "eslint-plugin-flowtype-errors/dist/format" {
  declare module.exports: any;
}

declare module "eslint-plugin-flowtype-errors/dist/index" {
  declare module.exports: any;
}

// Filename aliases
declare module "eslint-plugin-flowtype-errors/dist/collect.js" {
  declare module.exports: $Exports<
    "eslint-plugin-flowtype-errors/dist/collect"
  >;
}
declare module "eslint-plugin-flowtype-errors/dist/filter.js" {
  declare module.exports: $Exports<"eslint-plugin-flowtype-errors/dist/filter">;
}
declare module "eslint-plugin-flowtype-errors/dist/format.js" {
  declare module.exports: $Exports<"eslint-plugin-flowtype-errors/dist/format">;
}
declare module "eslint-plugin-flowtype-errors/dist/index.js" {
  declare module.exports: $Exports<"eslint-plugin-flowtype-errors/dist/index">;
}
declare module "eslint-plugin-flowtype-errors/index" {
  declare module.exports: $Exports<"eslint-plugin-flowtype-errors">;
}
declare module "eslint-plugin-flowtype-errors/index.js" {
  declare module.exports: $Exports<"eslint-plugin-flowtype-errors">;
}
