// flow-typed signature: 7150568b78b808926f81c1a651cb4f91
// flow-typed version: <<STUB>>/husky_v^0.14.3/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'husky'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "husky" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "husky/__tests__/index" {
  declare module.exports: any;
}

declare module "husky/bin/install" {
  declare module.exports: any;
}

declare module "husky/bin/uninstall" {
  declare module.exports: any;
}

declare module "husky/src/install" {
  declare module.exports: any;
}

declare module "husky/src/uninstall" {
  declare module.exports: any;
}

declare module "husky/src/utils/find-hooks-dir" {
  declare module.exports: any;
}

declare module "husky/src/utils/find-parent" {
  declare module.exports: any;
}

declare module "husky/src/utils/get-hook-script" {
  declare module.exports: any;
}

declare module "husky/src/utils/is-husky" {
  declare module.exports: any;
}

// Filename aliases
declare module "husky/__tests__/index.js" {
  declare module.exports: $Exports<"husky/__tests__/index">;
}
declare module "husky/bin/install.js" {
  declare module.exports: $Exports<"husky/bin/install">;
}
declare module "husky/bin/uninstall.js" {
  declare module.exports: $Exports<"husky/bin/uninstall">;
}
declare module "husky/src/install.js" {
  declare module.exports: $Exports<"husky/src/install">;
}
declare module "husky/src/uninstall.js" {
  declare module.exports: $Exports<"husky/src/uninstall">;
}
declare module "husky/src/utils/find-hooks-dir.js" {
  declare module.exports: $Exports<"husky/src/utils/find-hooks-dir">;
}
declare module "husky/src/utils/find-parent.js" {
  declare module.exports: $Exports<"husky/src/utils/find-parent">;
}
declare module "husky/src/utils/get-hook-script.js" {
  declare module.exports: $Exports<"husky/src/utils/get-hook-script">;
}
declare module "husky/src/utils/is-husky.js" {
  declare module.exports: $Exports<"husky/src/utils/is-husky">;
}
