// flow-typed signature: cd83c112b8869fe29d84a88fc9c28be0
// flow-typed version: <<STUB>>/lint-staged_v^6.0.0/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'lint-staged'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "lint-staged" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "lint-staged/src/calcChunkSize" {
  declare module.exports: any;
}

declare module "lint-staged/src/findBin" {
  declare module.exports: any;
}

declare module "lint-staged/src/generateTasks" {
  declare module.exports: any;
}

declare module "lint-staged/src/getConfig" {
  declare module.exports: any;
}

declare module "lint-staged/src/index" {
  declare module.exports: any;
}

declare module "lint-staged/src/printErrors" {
  declare module.exports: any;
}

declare module "lint-staged/src/resolveGitDir" {
  declare module.exports: any;
}

declare module "lint-staged/src/runAll" {
  declare module.exports: any;
}

declare module "lint-staged/src/runScript" {
  declare module.exports: any;
}

// Filename aliases
declare module "lint-staged/index" {
  declare module.exports: $Exports<"lint-staged">;
}
declare module "lint-staged/index.js" {
  declare module.exports: $Exports<"lint-staged">;
}
declare module "lint-staged/src/calcChunkSize.js" {
  declare module.exports: $Exports<"lint-staged/src/calcChunkSize">;
}
declare module "lint-staged/src/findBin.js" {
  declare module.exports: $Exports<"lint-staged/src/findBin">;
}
declare module "lint-staged/src/generateTasks.js" {
  declare module.exports: $Exports<"lint-staged/src/generateTasks">;
}
declare module "lint-staged/src/getConfig.js" {
  declare module.exports: $Exports<"lint-staged/src/getConfig">;
}
declare module "lint-staged/src/index.js" {
  declare module.exports: $Exports<"lint-staged/src/index">;
}
declare module "lint-staged/src/printErrors.js" {
  declare module.exports: $Exports<"lint-staged/src/printErrors">;
}
declare module "lint-staged/src/resolveGitDir.js" {
  declare module.exports: $Exports<"lint-staged/src/resolveGitDir">;
}
declare module "lint-staged/src/runAll.js" {
  declare module.exports: $Exports<"lint-staged/src/runAll">;
}
declare module "lint-staged/src/runScript.js" {
  declare module.exports: $Exports<"lint-staged/src/runScript">;
}
