// flow-typed signature: 4cdb0cdf7a4f8298c988f4b998e0ec70
// flow-typed version: <<STUB>>/glob_v^7.1.2/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'glob'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "glob" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "glob/common" {
  declare module.exports: any;
}

declare module "glob/glob" {
  declare module.exports: any;
}

declare module "glob/sync" {
  declare module.exports: any;
}

// Filename aliases
declare module "glob/common.js" {
  declare module.exports: $Exports<"glob/common">;
}
declare module "glob/glob.js" {
  declare module.exports: $Exports<"glob/glob">;
}
declare module "glob/sync.js" {
  declare module.exports: $Exports<"glob/sync">;
}
