// flow-typed signature: f1dc73683bf8b867984b9affc13ab2f6
// flow-typed version: <<STUB>>/prettier_v^1.9.2/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'prettier'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "prettier" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "prettier/bin/prettier" {
  declare module.exports: any;
}

declare module "prettier/parser-babylon" {
  declare module.exports: any;
}

declare module "prettier/parser-flow" {
  declare module.exports: any;
}

declare module "prettier/parser-graphql" {
  declare module.exports: any;
}

declare module "prettier/parser-markdown" {
  declare module.exports: any;
}

declare module "prettier/parser-parse5" {
  declare module.exports: any;
}

declare module "prettier/parser-postcss" {
  declare module.exports: any;
}

declare module "prettier/parser-typescript" {
  declare module.exports: any;
}

declare module "prettier/third-party" {
  declare module.exports: any;
}

// Filename aliases
declare module "prettier/bin/prettier.js" {
  declare module.exports: $Exports<"prettier/bin/prettier">;
}
declare module "prettier/index" {
  declare module.exports: $Exports<"prettier">;
}
declare module "prettier/index.js" {
  declare module.exports: $Exports<"prettier">;
}
declare module "prettier/parser-babylon.js" {
  declare module.exports: $Exports<"prettier/parser-babylon">;
}
declare module "prettier/parser-flow.js" {
  declare module.exports: $Exports<"prettier/parser-flow">;
}
declare module "prettier/parser-graphql.js" {
  declare module.exports: $Exports<"prettier/parser-graphql">;
}
declare module "prettier/parser-markdown.js" {
  declare module.exports: $Exports<"prettier/parser-markdown">;
}
declare module "prettier/parser-parse5.js" {
  declare module.exports: $Exports<"prettier/parser-parse5">;
}
declare module "prettier/parser-postcss.js" {
  declare module.exports: $Exports<"prettier/parser-postcss">;
}
declare module "prettier/parser-typescript.js" {
  declare module.exports: $Exports<"prettier/parser-typescript">;
}
declare module "prettier/third-party.js" {
  declare module.exports: $Exports<"prettier/third-party">;
}
