// flow-typed signature: f25cd1fafb5f89010b941e4f463c2332
// flow-typed version: <<STUB>>/eslint-config-prettier_v^2.9.0/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'eslint-config-prettier'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "eslint-config-prettier" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "eslint-config-prettier/bin/cli" {
  declare module.exports: any;
}

declare module "eslint-config-prettier/bin/validators" {
  declare module.exports: any;
}

declare module "eslint-config-prettier/flowtype" {
  declare module.exports: any;
}

declare module "eslint-config-prettier/react" {
  declare module.exports: any;
}

declare module "eslint-config-prettier/standard" {
  declare module.exports: any;
}

// Filename aliases
declare module "eslint-config-prettier/bin/cli.js" {
  declare module.exports: $Exports<"eslint-config-prettier/bin/cli">;
}
declare module "eslint-config-prettier/bin/validators.js" {
  declare module.exports: $Exports<"eslint-config-prettier/bin/validators">;
}
declare module "eslint-config-prettier/flowtype.js" {
  declare module.exports: $Exports<"eslint-config-prettier/flowtype">;
}
declare module "eslint-config-prettier/index" {
  declare module.exports: $Exports<"eslint-config-prettier">;
}
declare module "eslint-config-prettier/index.js" {
  declare module.exports: $Exports<"eslint-config-prettier">;
}
declare module "eslint-config-prettier/react.js" {
  declare module.exports: $Exports<"eslint-config-prettier/react">;
}
declare module "eslint-config-prettier/standard.js" {
  declare module.exports: $Exports<"eslint-config-prettier/standard">;
}
