// flow-typed signature: e09fdf0c8f99d06ce66ee89cc224dafe
// flow-typed version: <<STUB>>/@babel/core_v^7.0.0-beta.34/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   '@babel/core'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "@babel/core" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "@babel/core/lib/config/build-config-chain" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/caching" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/helpers/environment" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/index" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/loading/files/configuration" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/loading/files/index-browser" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/loading/files/index" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/loading/files/plugins" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/option-assertions" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/option-manager" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/options" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/plugin" {
  declare module.exports: any;
}

declare module "@babel/core/lib/config/removed" {
  declare module.exports: any;
}

declare module "@babel/core/lib/index" {
  declare module.exports: any;
}

declare module "@babel/core/lib/tools/build-external-helpers" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-ast-sync" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-ast" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-file-browser" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-file-sync-browser" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-file-sync" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-file" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform-sync" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transform" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/block-hoist-plugin" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/file/file" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/file/generate" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/index" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/normalize-file" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/normalize-opts" {
  declare module.exports: any;
}

declare module "@babel/core/lib/transformation/plugin-pass" {
  declare module.exports: any;
}

// Filename aliases
declare module "@babel/core/lib/config/build-config-chain.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/build-config-chain">;
}
declare module "@babel/core/lib/config/caching.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/caching">;
}
declare module "@babel/core/lib/config/helpers/environment.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/config/helpers/environment"
  >;
}
declare module "@babel/core/lib/config/index.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/index">;
}
declare module "@babel/core/lib/config/loading/files/configuration.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/config/loading/files/configuration"
  >;
}
declare module "@babel/core/lib/config/loading/files/index-browser.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/config/loading/files/index-browser"
  >;
}
declare module "@babel/core/lib/config/loading/files/index.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/config/loading/files/index"
  >;
}
declare module "@babel/core/lib/config/loading/files/plugins.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/config/loading/files/plugins"
  >;
}
declare module "@babel/core/lib/config/option-assertions.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/option-assertions">;
}
declare module "@babel/core/lib/config/option-manager.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/option-manager">;
}
declare module "@babel/core/lib/config/options.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/options">;
}
declare module "@babel/core/lib/config/plugin.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/plugin">;
}
declare module "@babel/core/lib/config/removed.js" {
  declare module.exports: $Exports<"@babel/core/lib/config/removed">;
}
declare module "@babel/core/lib/index.js" {
  declare module.exports: $Exports<"@babel/core/lib/index">;
}
declare module "@babel/core/lib/tools/build-external-helpers.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/tools/build-external-helpers"
  >;
}
declare module "@babel/core/lib/transform-ast-sync.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-ast-sync">;
}
declare module "@babel/core/lib/transform-ast.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-ast">;
}
declare module "@babel/core/lib/transform-file-browser.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-file-browser">;
}
declare module "@babel/core/lib/transform-file-sync-browser.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transform-file-sync-browser"
  >;
}
declare module "@babel/core/lib/transform-file-sync.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-file-sync">;
}
declare module "@babel/core/lib/transform-file.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-file">;
}
declare module "@babel/core/lib/transform-sync.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform-sync">;
}
declare module "@babel/core/lib/transform.js" {
  declare module.exports: $Exports<"@babel/core/lib/transform">;
}
declare module "@babel/core/lib/transformation/block-hoist-plugin.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transformation/block-hoist-plugin"
  >;
}
declare module "@babel/core/lib/transformation/file/file.js" {
  declare module.exports: $Exports<"@babel/core/lib/transformation/file/file">;
}
declare module "@babel/core/lib/transformation/file/generate.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transformation/file/generate"
  >;
}
declare module "@babel/core/lib/transformation/index.js" {
  declare module.exports: $Exports<"@babel/core/lib/transformation/index">;
}
declare module "@babel/core/lib/transformation/normalize-file.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transformation/normalize-file"
  >;
}
declare module "@babel/core/lib/transformation/normalize-opts.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transformation/normalize-opts"
  >;
}
declare module "@babel/core/lib/transformation/plugin-pass.js" {
  declare module.exports: $Exports<
    "@babel/core/lib/transformation/plugin-pass"
  >;
}
