// flow-typed signature: 5d86a43980fbbb1e5cf4b52e6e5b2bbc
// flow-typed version: <<STUB>>/performance-now_v^2.1.0/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'performance-now'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "performance-now" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "performance-now/lib/performance-now" {
  declare module.exports: any;
}

// Filename aliases
declare module "performance-now/lib/performance-now.js" {
  declare module.exports: $Exports<"performance-now/lib/performance-now">;
}
