// flow-typed signature: 17c07bb4303badb96f3e6004f8d20a1d
// flow-typed version: <<STUB>>/markdown-to-html_v0.0.13/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'markdown-to-html'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "markdown-to-html" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "markdown-to-html/lib/getopts" {
  declare module.exports: any;
}

declare module "markdown-to-html/lib/gfm-linkify" {
  declare module.exports: any;
}

declare module "markdown-to-html/lib/github-markdown" {
  declare module.exports: any;
}

declare module "markdown-to-html/lib/markdown" {
  declare module.exports: any;
}

declare module "markdown-to-html/test" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/routes/home" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/routes/index" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/routes/renderGithubMarkdown" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/routes/renderMarkdown" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/test" {
  declare module.exports: any;
}

declare module "markdown-to-html/web-demo/web" {
  declare module.exports: any;
}

// Filename aliases
declare module "markdown-to-html/index" {
  declare module.exports: $Exports<"markdown-to-html">;
}
declare module "markdown-to-html/index.js" {
  declare module.exports: $Exports<"markdown-to-html">;
}
declare module "markdown-to-html/lib/getopts.js" {
  declare module.exports: $Exports<"markdown-to-html/lib/getopts">;
}
declare module "markdown-to-html/lib/gfm-linkify.js" {
  declare module.exports: $Exports<"markdown-to-html/lib/gfm-linkify">;
}
declare module "markdown-to-html/lib/github-markdown.js" {
  declare module.exports: $Exports<"markdown-to-html/lib/github-markdown">;
}
declare module "markdown-to-html/lib/markdown.js" {
  declare module.exports: $Exports<"markdown-to-html/lib/markdown">;
}
declare module "markdown-to-html/test.js" {
  declare module.exports: $Exports<"markdown-to-html/test">;
}
declare module "markdown-to-html/web-demo/routes/home.js" {
  declare module.exports: $Exports<"markdown-to-html/web-demo/routes/home">;
}
declare module "markdown-to-html/web-demo/routes/index.js" {
  declare module.exports: $Exports<"markdown-to-html/web-demo/routes/index">;
}
declare module "markdown-to-html/web-demo/routes/renderGithubMarkdown.js" {
  declare module.exports: $Exports<
    "markdown-to-html/web-demo/routes/renderGithubMarkdown"
  >;
}
declare module "markdown-to-html/web-demo/routes/renderMarkdown.js" {
  declare module.exports: $Exports<
    "markdown-to-html/web-demo/routes/renderMarkdown"
  >;
}
declare module "markdown-to-html/web-demo/test.js" {
  declare module.exports: $Exports<"markdown-to-html/web-demo/test">;
}
declare module "markdown-to-html/web-demo/web.js" {
  declare module.exports: $Exports<"markdown-to-html/web-demo/web">;
}
