// flow-typed signature: 91ac714fe96282e901b75bc619c06aa8
// flow-typed version: <<STUB>>/eslint-plugin-prettier_v^2.4.0/flow_v0.60.1

/**
 * This is an autogenerated libdef stub for:
 *
 *   'eslint-plugin-prettier'
 *
 * Fill this stub out by replacing all the `any` types.
 *
 * Once filled out, we encourage you to share your work with the
 * community by sending a pull request to:
 * https://github.com/flowtype/flow-typed
 */

declare module "eslint-plugin-prettier" {
  declare module.exports: any;
}

/**
 * We include stubs for each file inside this npm package in case you need to
 * require those files directly. Feel free to delete any files that aren't
 * needed.
 */
declare module "eslint-plugin-prettier/eslint-plugin-prettier" {
  declare module.exports: any;
}

// Filename aliases
declare module "eslint-plugin-prettier/eslint-plugin-prettier.js" {
  declare module.exports: $Exports<
    "eslint-plugin-prettier/eslint-plugin-prettier"
  >;
}
